﻿param ($path)
function Get-GeoLocation {
    <#
        .SYNOPSIS
            Get current location.
        .DESCRIPTION
            This function tries to get the latitude and longitude coordinates of the current location,
            and will look up those coordinates on OpenStreetMap to get address information.
        .EXAMPLE
            Get-GeoLocation
            Will try to get address information about your current location.
        .EXAMPLE
            Get-GeoLocation -LatLon
            Will return the Latitude and Longitude of your current location.
        .NOTES
            Author: Øyvind Kallstad
            Version: 1.0
            Date: 09.08.2016
        .LINK
            https://communary.net/
            
    #>
    [CmdletBinding()]
    $NumTries = 4
    $SleepBetweenTries = 1000
    try {
        Add-Type –AssemblyName 'System.Device'
    }

    catch {
        Write-Warning 'Unable to load the needed assembly (System.Device).'
        break
    }

    $watcher = New-Object –TypeName System.Device.Location.GeoCoordinateWatcher –ArgumentList 'High'
    [void]$watcher.TryStart($true, [TimeSpan]::FromMilliseconds(1000))

    $count = 0

    do {
        $count++
        Start-Sleep –Milliseconds $SleepBetweenTries
    } while (($watcher.Position.Location.IsUnknown) -or ($count -ge $numTries))

    if ($watcher.Position.Location.IsUnknown) {
        Write-Warning 'Couldn''t get coordinates.'
    }

    else {
        $coord = $watcher.Position.Location

        if (-not ($LatLon)) {
            $url = "http://nominatim.openstreetmap.org/reverse?format=json&lat=$($coord.Latitude.ToString().Replace(',','.'))&lon=$($coord.Longitude.ToString().Replace(',','.'))&zoom=18&addressdetails=1"
    
            try {
                $result = Invoke-RestMethod –Uri $url
                Write-Output ($result.address)
            }

            catch {
                Write-Warning $_.Exception.Message
            }
        }

        else {
            Write-Output ([PSCustomObject][Ordered] @{
                Latitude = $coord.Latitude
                Longitude = $coord.Longitude
            })
        }
    }
}

New-Alias –Name whereami –Value Get-GeoLocation –Force

<#Get-GeoLocation | write-output#>
Get-GeoLocation > $path\address.txt

